﻿/*
 * Created by SharpDevelop.
 * User: Marco José
 * Date: 29-12-2006
 * Time: 19:27
 */

using System;
using MySql.Data.MySqlClient;

namespace adav
{
	
	/// <summary>
	/// Description of ICartao.
	/// </summary>
	public interface ICartao
	{
		/// <summary>
		/// criacao de um novo cartao
		/// </summary>
		/// <param name="num_telemovel"></param>
		/// <param name="tipo_cartao"></param>
		/// <param name="saldo_inicial"></param>
		/// <param name="comissao_operador"></param>
		/// <param name="id_tarifario"></param>
		/// <returns> id cartao (string) </returns>
		OperationStatus CriarCartao(string username, string password, string num_telemovel, char tipo_cartao, double saldo_inicial, char estado_cartao, int id_tarifario);
		
		/// <summary>
		/// alterar o plano tarifario de um cartao
		/// </summary>
		/// <param name="num_telemovel"></param>
		/// <param name="id_tarifario"></param>
		/// <returns></returns>
		OperationStatus AlterarPlanoTarifario(string username, string password, string num_telemovel, int id_tarifario);
		
		/// <summary>
		/// cancelamento de um cartao
		/// </summary>
		/// <param name="num_telemovel"></param>
		/// <returns></returns>
		OperationStatus CancelarCartao(string username, string password, string num_telemovel);
		
		/// <summary>
		/// obter todos os tipos de cartao existentes (recarregavel, assinatura)
		/// </summary>
		/// <param name="username"></param>
		/// <param name="password"></param>
		/// <returns> os tipos existentes </returns>
		string [] ObterTiposCartao(string username, string password);
		
		/// <summary>
		/// carregamento do cartao recarregável
		/// </summary>
		/// <param name="username"></param>
		/// <param name="password"></param>
		/// <param name="num_telemovel"></param>
		/// <param name="montante"></param>
		/// <returns> estado (se carregou com sucesso ou não) </returns>
		OperationStatus CarregarCartao(string username, string password, string num_telemovel, double montante);
		
	}

	/// <summary>
	/// Description of Cartao.
	/// </summary>
	public class Cartao :ICartao 
	{
		
		public Cartao()
		{
		}
		
		public OperationStatus CriarCartao(string username, string password, string num_telemovel, char tipo_cartao, double saldo_inicial, char estado_cartao, int id_tarifario)
		{
			OperationStatus status = DBHelper.ValidateUser(username,password);
			if(status != OperationStatus.OK)
				return status;
			
			MySqlConnection conn = DBHelper.getConnection();
			string sql = "INSERT INTO cartao (num_telemovel,tipo_cartao,saldo,estado_cartao,id_tarifario) VALUES (?num_telemovel,?tipo_cartao,?saldo,?estado_cartao,?id_tarifario);";
			try {
				MySqlCommand cmd = new MySqlCommand(sql,conn);
				cmd.Parameters.Add("?num_telemovel",num_telemovel);
				cmd.Parameters.Add("?tipo_cartao",tipo_cartao.ToString());
				cmd.Parameters.Add("?saldo",saldo_inicial);
				cmd.Parameters.Add("?estado_cartao",estado_cartao.ToString());
				cmd.Parameters.Add("?id_tarifario",id_tarifario);
				
				conn.Open();
				int ret = cmd.ExecuteNonQuery();
				status = OperationStatus.CARTAO_CRIADO;
			}
			catch(MySqlException ex ){
				System.Console.WriteLine("EXCEPÇÃO na criação do 'Cartão': ");
				Console.WriteLine(ex);
				status = OperationStatus.ERROR;
			}
			finally {
				DBHelper.closeConnection(conn);
			}
			
			return status;
		}
		
		public OperationStatus AlterarPlanoTarifario(string username, string password, string num_telemovel, int id_tarifario)
		{
			//
			OperationStatus status = DBHelper.ValidateUser(username,password);
			if(status != OperationStatus.OK)
				return status;
			
			MySqlConnection conn = DBHelper.getConnection();
			string sql = "update cartao set id_tarifario = ?id_tarifario where num_telemovel = ?num_telemovel;";
			try {
				MySqlCommand cmd = new MySqlCommand(sql,conn);
				cmd.Parameters.Add("?id_tarifario",id_tarifario);
				cmd.Parameters.Add("?num_telemovel",num_telemovel);
				
				conn.Open();
				int ret = cmd.ExecuteNonQuery();

				// obter a comissao do tarifario
				string sql2 = "select comissao from tarifario where id_tarifario = ?id_tarifario ;";
				MySqlCommand cmd2 = new MySqlCommand(sql2,conn);
				cmd2.Parameters.Add("?id_tarifario",id_tarifario);
				
				MySqlDataReader dr = cmd2.ExecuteReader();
				if(dr.Read()){
					// obter a comissao total recebida pelo funcionario
					double com = dr.GetDouble(0);
					string sql3 = "select comissao_total FROM funcionario " +
						"where username like ?username and password like ?password ; ";
					MySqlCommand cmd3 = new MySqlCommand(sql3,conn);
					cmd3.Parameters.Add("?username",username);
					cmd3.Parameters.Add("?password",password);
					DBHelper.closeReader(dr);
					MySqlDataReader dr2 = cmd3.ExecuteReader();
					if(dr2.Read()) {
						// actualizar a comissao total
						com += dr2.GetDouble(0);
						string sql4 = "update funcionario set comissao_total = ?com_total " +
							"where username like ?username and password like ?password ; ";
						MySqlCommand cmd4 = new MySqlCommand(sql4,conn);
						cmd4.Parameters.Add("?com_total",com);
						cmd4.Parameters.Add("?username",username);
						cmd4.Parameters.Add("?password",password);
						DBHelper.closeReader(dr2);
						cmd4.ExecuteNonQuery();
					}
					
				}
				else {
					DBHelper.closeReader(dr);
				}
				
				status = OperationStatus.PLANO_TARIFARIO_ALTERADO;
			}
			catch(MySqlException ex ){
				System.Console.WriteLine("EXCEPÇÃO na alteração do Plano tarifário: ");
				Console.WriteLine(ex);
				status = OperationStatus.ERROR;
			}
			finally {
				DBHelper.closeConnection(conn);
			}
			
			return status;
		}
		
		public OperationStatus CancelarCartao(string username, string password, string num_telemovel)
		{
			OperationStatus status = DBHelper.ValidateUser(username,password);
			if(status != OperationStatus.OK)
				return status;
			
			MySqlConnection conn = DBHelper.getConnection();
			string sql = "UPDATE cartao SET estado_cartao = ?estado_cartao WHERE num_telemovel = ?num_telemovel;";
			try {
				MySqlCommand cmd = new MySqlCommand(sql,conn);
				cmd.Parameters.Add("?estado_cartao",EstadoCartao.DESACTIVADO.ToString());
				cmd.Parameters.Add("?num_telemovel",num_telemovel);
				
				conn.Open();
				int ret = cmd.ExecuteNonQuery();
				status = OperationStatus.CARTAO_CANCELADO;
			}
			catch(MySqlException ex ){
				System.Console.WriteLine("EXCEPÇÃO no cancelamento do 'Cartão': ");
				Console.WriteLine(ex);
				status = OperationStatus.ERROR;
			}
			finally {
				DBHelper.closeConnection(conn);
			}
			
			return status;
		}
		
		public string [] ObterTiposCartao(string username, string password)
		{
			OperationStatus status = DBHelper.ValidateUser(username,password);
			if(status != OperationStatus.OK)
			{
				return null;
			}
			string []ret = new string[2];
			ret[0]=TipoCartao.PRE_PAGO_STRING;
			ret[1]=TipoCartao.POS_PAGO_STRING;
			return ret;
		}
		
		public OperationStatus CarregarCartao(string username, string password, string num_telemovel, double montante)
		{
			//select saldo from cartao where num_telemovel like 's' and tipo_cartao like 'R'
			OperationStatus status = DBHelper.ValidateUser(username,password);
			if(status != OperationStatus.OK)
				return status;
			
			MySqlConnection conn = DBHelper.getConnection();
			string sql = "select saldo from cartao " +
				"where num_telemovel like ?num_telemovel and tipo_cartao like ?tipo_cartao and estado_cartao = ?estado;" ;
			string sql2 = "update cartao set saldo = ?saldo where num_telemovel like ?num_telemovel ; ";
			try {
				double oldSaldo = 0;
				MySqlCommand cmd = new MySqlCommand(sql,conn);
				cmd.Parameters.Add("?num_telemovel",num_telemovel);
				cmd.Parameters.Add("?tipo_cartao",TipoCartao.PRE_PAGO_DBVALUE.ToString());
				cmd.Parameters.Add("?estado",EstadoCartao.ACTIVADO.ToString());

				conn.Open();

				MySqlDataReader dr = cmd.ExecuteReader();
				if(dr.Read()){
					oldSaldo = dr.GetDouble(0);
					
					MySqlCommand cmd2 = new MySqlCommand(sql2,conn);
					cmd2.Parameters.Add("?saldo",oldSaldo + montante);
					cmd2.Parameters.Add("?num_telemovel", num_telemovel);
					// é preciso fechar o datareader antes de executar um novo comando com a mm conexao
					DBHelper.closeReader(dr);
					int x = cmd2.ExecuteNonQuery();
					status = OperationStatus.CARREGAMENTO_EFECTUADO;
				}
				else 
				{
					DBHelper.closeReader(dr);
					status = OperationStatus.CARREGAMENTO_FALHOU;
				}
				
			}
			catch(MySqlException ex ){
				System.Console.WriteLine("EXCEPÇÃO no carregamento do 'Cartão': ");
				Console.WriteLine(ex);
				status = OperationStatus.ERROR;
			}
			finally {
				DBHelper.closeConnection(conn);
			}
			
			return status;
		}
		
	}
}
