﻿/*
 * Created by SharpDevelop.
 * User: Marco José
 * Date: 08-01-2007
 * Time: 22:16
 */

using System;
using MySql.Data.MySqlClient;

namespace adav
{

	/// <summary>
	/// Description of Chamada.
	/// </summary>
	public interface IChamada
	{
		/// <summary>
		/// Simulação de inicio de uma chamada
		/// </summary>
		/// <param name="id_cliente"></param>
		/// <param name="num_telemovel"></param>
		/// <returns> id chamada </returns>
		OperationStatus InicioChamada(string username, string password, string num_telemovel, string num_destino);
		
		/// <summary>
		/// Simulação do fim de uma chamada
		/// </summary>
		/// <param name="id_chamada"></param>
		/// <returns> duracao chamada </returns>
		OperationStatus FimChamada(string username, string password, string num_telemovel, string num_destino);
		
		/// <summary>
		/// obter o estado de uma chamada a partir de um ID
		/// </summary>
		/// <param name="id_chamada"></param>
		/// <returns> estado da chamada </returns>
//		EstadoChamada GetEstadoChamada(string username, string password, int id_chamada);
		
	}
	
	/// <summary>
	/// Description of Chamada.
	/// </summary>
	public class Chamada : IChamada 
	{
		public Chamada()
		{
		}
		
		public OperationStatus InicioChamada(string username, string password, string num_telemovel, string num_destino)
		{
			OperationStatus status = DBHelper.ValidateUser(username,password);
			if(status != OperationStatus.OK)
				return status;

			if(num_telemovel==num_destino)
				return OperationStatus.CHAMADA_TERMINADA;
			
			if(DBHelper.GetEstadoCartao(num_telemovel)==EstadoCartao.DESACTIVADO.ToString())
				return OperationStatus.ERROR;
			
			MySqlConnection conn = DBHelper.getConnection();
			string sql = "insert into chamada " +
				"(num_telemovel, num_destino, data_inicio, estado_chamada) " +
				"values (?num_telemovel,?num_destino,?data_inicio, ?estado) ;";
			string sql2 = "select saldo,tipo_cartao from cartao where num_telemovel = ?num_telemovel ; ";
			try {
				conn.Open();
				MySqlCommand cmd2 = new MySqlCommand(sql2,conn);
				cmd2.Parameters.Add("?num_telemovel",num_telemovel);
				MySqlDataReader dr = cmd2.ExecuteReader();
				if(dr.Read()) {
					double saldo = dr.GetDouble(0);
					string tipo = dr.GetString(1);
					if((saldo > 0 && tipo == TipoCartao.PRE_PAGO_DBVALUE.ToString()) || tipo == TipoCartao.POS_PAGO_DBVALUE.ToString()) {
						MySqlCommand cmd = new MySqlCommand(sql,conn);
						cmd.Parameters.Add("?num_telemovel",num_telemovel);
						cmd.Parameters.Add("?num_destino",num_destino);
						cmd.Parameters.Add("?data_inicio",DBHelper.getMySQLDateTime(DateTime.Now));
						cmd.Parameters.Add("?estado",EstadoChamada.ACTIVO.ToString());
						DBHelper.closeReader(dr);
						int ret = cmd.ExecuteNonQuery();
						status = OperationStatus.CHAMADA_INICIADA;
					}
					else {
						status = OperationStatus.CHAMADA_TERMINADA;
					}
				}
				
			}
			catch(MySqlException ex ){
				System.Console.WriteLine("EXCEPÇÃO no início da 'Chamada': ");
				Console.WriteLine(ex);
				status = OperationStatus.ERROR;
			}
			finally {
				DBHelper.closeConnection(conn);
			}
			
			return status;
		}
		
		public OperationStatus FimChamada(string username, string password, string num_telemovel, string num_destino)
		{
			if(num_telemovel==num_destino)
				return OperationStatus.CHAMADA_TERMINADA;
			
			OperationStatus status = DBHelper.ValidateUser(username,password);
			if(status != OperationStatus.OK)
				return status;
			
			MySqlConnection conn = DBHelper.getConnection();
			string sql1 = "select max(id_chamada) from chamada " +
				"where num_telemovel like ?num_telemovel and num_destino like ?num_destino " +
				"and estado_chamada = ?estado_chamada;";
			string sql = "select data_inicio from chamada where id_chamada = ?id_chamada ;";
			string sql2 = "update chamada set data_fim = ?data_fim, duracao = ?duracao, custo_chamada = ?custo_chamada, " +
				"estado_chamada = ?estado_chamada where id_chamada = ?id_chamada ;";
			string sql3 = "select tipo_cartao, saldo from cartao where num_telemovel = ?num_telemovel ; ";
			string sql4 = "update cartao set saldo = ?saldo where num_telemovel = ?num_telemovel ; ";
			try {
				MySqlCommand cmd1 = new MySqlCommand(sql1,conn);
				cmd1.Parameters.Add("?num_telemovel",num_telemovel);
				cmd1.Parameters.Add("?num_destino",num_destino);
				cmd1.Parameters.Add("?estado_chamada",EstadoChamada.ACTIVO.ToString());
				conn.Open();
				MySqlDataReader dr1 = cmd1.ExecuteReader();
				if(dr1.Read()) {
					int id_chamada = dr1.GetInt32(0);
					MySqlCommand cmd = new MySqlCommand(sql,conn);
					cmd.Parameters.Add("?id_chamada",id_chamada);
					cmd.Parameters.Add("?num_destino",num_destino);
					cmd.Parameters.Add("?estado_chamada",EstadoChamada.ACTIVO.ToString());
					DBHelper.closeReader(dr1);
					MySqlDataReader dr = cmd.ExecuteReader();
					if(dr.Read()) {
						DateTime ini_chamada = dr.GetDateTime(0);
						DateTime fim_chamada = DateTime.Now;
						// diferenca entre horas
						TimeSpan ts = fim_chamada.Subtract(ini_chamada); 
	//					Console.WriteLine(ts.Ticks);
						
						DBHelper.closeReader(dr);
						double custo_chamada = ObterCustoChamada(conn,ini_chamada,ts);
						
						MySqlCommand cmd2 = new MySqlCommand(sql2,conn);
						cmd2.Parameters.Add("?data_fim",DBHelper.getMySQLDateTime(fim_chamada));
						cmd2.Parameters.Add("?duracao",getDuracaoSegundos(ts));
						cmd2.Parameters.Add("?custo_chamada",custo_chamada); //falta obter custo da chamada
						cmd2.Parameters.Add("?estado_chamada",EstadoChamada.DESLIGADO.ToString());
						cmd2.Parameters.Add("?id_chamada",id_chamada);
						cmd2.ExecuteNonQuery();
						
						MySqlCommand cmd3 = new MySqlCommand(sql3,conn);
						cmd3.Parameters.Add("?num_telemovel",num_telemovel);
						
						MySqlDataReader dr2 = cmd3.ExecuteReader();
						if(dr2.Read()) {
							string tipo = dr2.GetString(0);
							double saldo_antigo = dr2.GetDouble(1);
							double novo_saldo = -1;
							if(tipo == TipoCartao.PRE_PAGO_DBVALUE.ToString())
								novo_saldo = saldo_antigo - custo_chamada;
							if(tipo == TipoCartao.POS_PAGO_DBVALUE.ToString())
								novo_saldo = saldo_antigo + custo_chamada;
							
							MySqlCommand cmd4 = new MySqlCommand(sql4,conn);
							cmd4.Parameters.Add("?saldo",novo_saldo);
							cmd4.Parameters.Add("?num_telemovel",num_telemovel);
							
							DBHelper.closeReader(dr2);
							cmd4.ExecuteNonQuery();
							status = OperationStatus.CHAMADA_TERMINADA;
						}
						else {
							DBHelper.closeReader(dr2);
							status = OperationStatus.ERROR;
						}
					}
					else {
						DBHelper.closeReader(dr);
						status = OperationStatus.OK;
					}
				}
				else {
					DBHelper.closeReader(dr1);
					status = OperationStatus.ERROR;
				}
			}
			catch(MySqlException ex ){
				System.Console.WriteLine("EXCEPÇÃO no fim da 'Chamada': ");
				Console.WriteLine(ex);
				status = OperationStatus.ERROR;
			}
			finally {
				DBHelper.closeConnection(conn);
			}
			
			return status;
		}
		
		private double ObterCustoChamada(MySqlConnection conn, DateTime inicio, TimeSpan ts) {
			double ret = 0;
			long total_segundos = getDuracaoSegundos(ts);
			string hours = ts.Hours<10?"0"+ts.Hours:ts.Hours.ToString();
			string minutes = ts.Minutes<10?"0"+ts.Minutes:ts.Minutes.ToString();
			string seconds = ts.Seconds<10?"0"+ts.Seconds:ts.Seconds.ToString();
			string total = hours + minutes + seconds;
			
			string hour_period = (inicio.Hour<10?"0"+inicio.Hour:inicio.Hour.ToString()) + "00";
			string sql = "select id_periodo from periodo where hora_inicio like ?hora_inicio and id_ciclo = ?id_ciclo ;";
			MySqlCommand cmd = new MySqlCommand(sql, conn);
			cmd.Parameters.Add("?hora_inicio",hour_period);
			cmd.Parameters.Add("?id_ciclo",DBHelper.getIdForDayOfWeek(inicio.DayOfWeek));
			MySqlDataReader dr = cmd.ExecuteReader();
			int id_periodo = -1;
			if(dr.Read()){
				id_periodo = dr.GetInt32(0);
//select t.custo_unidade, t.unidades_minimas, u.unidade from tarifario_detalhe t, unidade u where t.id_periodo = 16 and t.id_unidade = u.id_unidade;
				//select id_tarifario, id_unidade, custo_unidade, unidades_minimas from tarifario_detalhe where id_periodo = 16;
//				sql = "select id_tarifario, id_unidade, custo_unidade, unidades_minimas " +
//					"from tarifario_detalhe where id_periodo = ?id_periodo ;";
				sql = "select t.custo_unidade, t.unidades_minimas, u.unidade " +
					"from tarifario_detalhe t, unidade u " +
					"where t.id_periodo = ?id_periodo and t.id_unidade = u.id_unidade;";
				MySqlCommand cmd2 = new MySqlCommand(sql,conn);
				cmd2.Parameters.Add("?id_periodo",16); 
				
				int unidade = -1;
				int unidades_minimas = -1;
				double custo_unidade = -1;
				DBHelper.closeReader(dr);
				MySqlDataReader dr2 = cmd2.ExecuteReader();
				if(dr2.Read()) {
					unidades_minimas = dr2.GetInt32(1);
					custo_unidade = dr2.GetDouble(0);
					unidade = dr2.GetInt32(2);
				}
				DBHelper.closeReader(dr2);
				
				if(total_segundos < unidades_minimas) {
					ret = unidades_minimas * custo_unidade;
				}
				else {
					// arredondamento por causa de taxacao (tarifario)
					long resto = total_segundos%unidade;
					if(resto!=0){
						resto = (unidade-resto);
					}
					total_segundos += resto;
					
					ret = custo_unidade * total_segundos;
				}
			}
			else {
				DBHelper.closeReader(dr);
			}
			
			return ret;
		}
		
		private long getDuracaoSegundos(TimeSpan ts) {
			long ret = ts.Seconds;
			ret += (ts.Minutes * 60);
			ret += (ts.Hours * 60 * 60);
			ret += (ts.Days * 24 * 60 * 60);			
			return ret;
		}

	}
}
