﻿/*
 * Created by SharpDevelop.
 * User: Marco José
 * Date: 08-01-2007
 * Time: 22:30
 */

using System;
using System.Data;
using MySql.Data.MySqlClient;

namespace adav
{

	/// <summary>
	/// Description of ICliente.
	/// </summary>
	public interface ICliente
	{
		/// <summary>
		/// criacao de um novo cliente 
		/// </summary>
		/// <param name="nome_cliente"></param>
		/// <returns> id cliente </returns>
		OperationStatus CriarCliente(string username, string password, string nome, string morada, string contribuinte);
		
		/// <summary>
		/// atribuir um cartao a um cliente 
		/// </summary>
		/// <param name="id_cliente"></param>
		/// <param name="num_telemovel"></param>
		OperationStatus AssociarCartaoCliente(string username, string password, int id_cliente, string num_telemovel);
		
		/// <summary>
		/// alteracao dos dados de um cliente 
		/// </summary>
		/// <param name="id_cliente"></param>
		/// <param name="nome_cliente"></param>
		/// <returns> id cliente </returns>
		OperationStatus AlterarDadosCliente(string username, string password, int id_cliente, string nome, string morada, string contribuinte);
		
		/// <summary>
		/// gera factura para todos os cartoes de um cliente 
		/// </summary>
		/// <param name="id_cliente"></param>
		/// <returns> dataset com a respectiva factura </returns>
		DataSet GerarFacturaPorCliente(string username, string password, int id_cliente);
		
		/// <summary>
		/// gera factura para um determinado cartao
		/// </summary>
		/// <param name="id_cliente"></param>
		/// <param name="num_telemovel"></param>
		/// <returns> dataset com a respectiva factura </returns>
		DataSet GerarFacturaPorCartao(string username, string password, string num_telemovel);
	}

	/// <summary>
	/// Description of Cliente.
	/// </summary>
	public class Cliente : ICliente
	{
		public Cliente()
		{
		}
		
		public OperationStatus CriarCliente(string username, string password, string nome, string morada, string contribuinte)
		{
			OperationStatus status = DBHelper.ValidateUser(username,password);
			if(status != OperationStatus.OK)
				return status;
			
			MySqlConnection conn = DBHelper.getConnection();
			string sql = "INSERT INTO cliente (nome,morada,num_contribuinte) VALUES (?nome,?morada,?nc);";
			
			try {
				MySqlCommand cmd = new MySqlCommand(sql,conn);
				cmd.Parameters.Add("?nome",nome);
				cmd.Parameters.Add("?morada",morada);
				cmd.Parameters.Add("?nc",contribuinte);
				
				conn.Open();
				int ret = cmd.ExecuteNonQuery();
				status = OperationStatus.CLIENTE_CRIADO;
			}
			catch(MySqlException ex ){
				System.Console.WriteLine("EXCEPÇÃO na inserção do 'Cliente': ");
				Console.WriteLine(ex);
				status = OperationStatus.ERROR;
			}
			finally {
				DBHelper.closeConnection(conn);
			}
			
			return status;
		}
		
		public OperationStatus AssociarCartaoCliente(string username, string password, int id_cliente, string num_telemovel)
		{
			OperationStatus status = DBHelper.ValidateUser(username,password);
			if(status != OperationStatus.OK)
				return status;
			
			MySqlConnection conn = DBHelper.getConnection();
			string sql = "insert into cartao_cliente values (?num_telemovel,?id_cliente);";
			
			try {
				MySqlCommand cmd = new MySqlCommand(sql,conn);
				cmd.Parameters.Add("?num_telemovel",num_telemovel);
				cmd.Parameters.Add("?id_cliente",id_cliente);
				
				conn.Open();
				int ret = cmd.ExecuteNonQuery();
				
				status = OperationStatus.CARTAO_ASSOCIADO;
			}
			catch(MySqlException ex ){
				System.Console.WriteLine("EXCEPÇÃO na associação do 'Cliente': ");
				Console.WriteLine(ex);
				status = OperationStatus.ERROR;
			}
			finally {
				DBHelper.closeConnection(conn);
			}
			
			return status;
		}
		
		public OperationStatus AlterarDadosCliente(string username, string password, int id_cliente, string nome, string morada, string contribuinte)
		{
			OperationStatus status = DBHelper.ValidateUser(username,password);
			if(status != OperationStatus.OK)
				return status;
			
			MySqlConnection conn = DBHelper.getConnection();
			string sql = "update cliente set nome = ?nome, morada=?morada,num_contribuinte=?nc where id_cliente = ?id;";
			
			try {
				MySqlCommand cmd = new MySqlCommand(sql,conn);
				cmd.Parameters.Add("?nome",nome);
				cmd.Parameters.Add("?morada",morada);
				cmd.Parameters.Add("?nc",contribuinte);
				cmd.Parameters.Add("?id",id_cliente);
				
				conn.Open();
				int ret = cmd.ExecuteNonQuery();
				status = OperationStatus.DADOS_ALTERADOS;
			}
			catch(MySqlException ex ){
				System.Console.WriteLine("EXCEPÇÃO na actualização do 'Cliente': ");
				Console.WriteLine(ex);
				status = OperationStatus.ERROR;
			}
			finally {
				DBHelper.closeConnection(conn);
			}
			
			return status;
		}
		
		public DataSet GerarFacturaPorCliente(string username, string password, int id_cliente)
		{
			OperationStatus status = DBHelper.ValidateUser(username,password);
			if(status != OperationStatus.OK)
				return null;
			
			MySqlConnection conn = DBHelper.getConnection();
			conn.Open();
			string sql = "select num_telemovel from cartao_cliente where id_cliente = ?id_cliente ; ";
			try {
				MySqlCommand cmd1 = new MySqlCommand(sql,conn);
				cmd1.Parameters.Add("?id_cliente",id_cliente);
				MySqlDataReader dr = cmd1.ExecuteReader();
				
				System.Collections.ArrayList al = new System.Collections.ArrayList();
				while(dr.Read()){
					al.Add(dr.GetString(0));
				}
				DBHelper.closeReader(dr);
				for(int i=0; i<al.Count; i++) {
					GerarFacturaPorCartao(conn,username,password,al[i].ToString());
				}
			}
			catch(MySqlException ex ){
				System.Console.WriteLine("EXCEPÇÃO na geraçao da factura para o 'Cartão': ");
				Console.WriteLine(ex);
			}
			finally {
				DBHelper.closeConnection(conn);
			}
			
			return null;
		}
		
		public DataSet GerarFacturaPorCartao(string username, string password, string num_telemovel)
		{
			OperationStatus status = DBHelper.ValidateUser(username,password);
			if(status != OperationStatus.OK)
				return null;
			
			MySqlConnection conn = DBHelper.getConnection();
			conn.Open();
			
			return GerarFacturaPorCartao(conn,username,password,num_telemovel);
		}
		
		private DataSet GerarFacturaPorCartao(MySqlConnection conn, string username, string password, string num_telemovel) 
		{
			if(conn.State != ConnectionState.Open)
				conn.Open();
			string sql = "update cartao set saldo = 0 where num_telemovel like ?num_telemovel and tipo_cartao = ?tipo_cartao ;";
			string sql1 = "select saldo from cartao where num_telemovel like ?num_telemovel and tipo_cartao = ?tipo_cartao ;";
			try {
				MySqlCommand cmd1 = new MySqlCommand(sql1,conn);
				cmd1.Parameters.Add("?num_telemovel",num_telemovel);
				cmd1.Parameters.Add("?tipo_cartao",TipoCartao.POS_PAGO_DBVALUE.ToString());
				MySqlDataReader dr = cmd1.ExecuteReader();
				double saldo = -1;
				if(dr.Read()){
					saldo = dr.GetDouble(0);
				}
				DBHelper.closeReader(dr);
				MySqlCommand cmd = new MySqlCommand(sql,conn);
				cmd.Parameters.Add("?num_telemovel",num_telemovel);
				cmd.Parameters.Add("?tipo_cartao",TipoCartao.POS_PAGO_DBVALUE.ToString());
				
				int ret = cmd.ExecuteNonQuery();
			}
			catch(MySqlException ex ){
				System.Console.WriteLine("EXCEPÇÃO na geraçao da factura para o 'Cartão': ");
				Console.WriteLine(ex);
			}
			finally {
				DBHelper.closeConnection(conn);
			}
			
			return null;
		}
	}
}
