﻿/*
 * Created by SharpDevelop.
 * User: Marco José
 * Date: 10-01-2007
 * Time: 22:38
 */

using System;

using MySql.Data.MySqlClient;

namespace adav
{
	/// <summary>
	/// Description of DBHelper.
	/// </summary>
	public class DBHelper
	{
		private static string CONNECTION_STRING = "server=127.0.0.1;uid=adav;pwd=adav;database=adav;";
		
		public DBHelper()
		{
		}
		
		public static MySqlConnection getConnection() {
			MySqlConnection conn = new MySqlConnection(DBHelper.CONNECTION_STRING);
			return conn;
		}
		
		public static void closeConnection(MySqlConnection conn) {
			if (conn.State == System.Data.ConnectionState.Open) {
				conn.Close();
			}
		}
		
		public static void closeReader(MySqlDataReader dr) {
			if(! dr.IsClosed) 
				dr.Close();
		}
		
		public static string getMySQLDateTime(DateTime dateTime) {
			string dateTimeString = dateTime.ToString("s");
			return dateTimeString.Substring(0,10) + " " + dateTimeString.Substring(11);
		}
		
		public static int getIDPeriodoByCiclo(MySqlConnection conn, int id_ciclo, int hora_inicio, int hora_fim) {
			int ret = -1;
			if(conn.State !=System.Data.ConnectionState.Open) {
				conn.Open();
			}
			
			string sql = "select id_periodo from periodo where id_ciclo = ?ciclo " +
				"and hora_inicio like ?ini and hora_fim like ?fim";
			MySqlCommand cmd = new MySqlCommand(sql,conn);
			cmd.Parameters.Add("?ciclo",id_ciclo);
			cmd.Parameters.Add("?ini",get4DigitString(hora_inicio));
			cmd.Parameters.Add("?fim",get4DigitString(hora_fim));
			
			MySqlDataReader dr = cmd.ExecuteReader();
			if(dr.Read()){
				ret = dr.GetInt32(0);
			}
			closeReader(dr);
			
			return ret;
		}
		
		private static string get4DigitString(int digit) {
			string ret = "";
			
			if(digit<10)
				ret +="0";
			
			ret += digit;
			ret += "00";
			
			return ret;
		}
		
		public static OperationStatus ValidateUser(string username, string password)
		{
			OperationStatus status = OperationStatus.OK;
			MySqlConnection conn = DBHelper.getConnection();

			try
			{
				//
				// criar objecto de conexão à base de dados e abrir a conexão
				//
				conn.Open();

				MySqlCommand cmd = new MySqlCommand("SELECT * FROM funcionario where username=?username and password=?password;",conn);
				cmd.Parameters.Add("?username",username);
				cmd.Parameters.Add("?password",password);
				
				MySqlDataReader dr = cmd.ExecuteReader();
				
				if (dr.HasRows)
					status = OperationStatus.OK;
				else
					status = OperationStatus.LOGIN_INVALIDO;

				dr.Close();
			}
			catch(MySqlException ex)
			{
				//
				// tratar a excepção!!!!
				//
				System.Console.WriteLine("EXCEPÇÃO na validação do 'User': ");
				Console.WriteLine(ex);
				status = OperationStatus.ERROR;
			}
			finally
			{
				//
				// fechar a conexão
				//
				DBHelper.closeConnection(conn);
			}
			
			return status;
		}
		
		public static TiposUtilizador GetUserType(string username, string password)
		{
			TiposUtilizador status = TiposUtilizador.NONE;
			MySqlConnection conn = DBHelper.getConnection();

			try
			{
				conn.Open();
				
				string sql = "select tipo_funcionario from funcionario " +
					"where username like ?username and password like ?password ;";
				MySqlCommand cmd = new MySqlCommand(sql,conn);
				cmd.Parameters.Add("?username",username);
				cmd.Parameters.Add("?password",password);
				
				MySqlDataReader dr = cmd.ExecuteReader();
				
				if (dr.Read()) {
					string tipo = dr.GetChar(0).ToString();
					if(tipo=="O")
						status = TiposUtilizador.OPERADOR;
					if(tipo=="S")
						status = TiposUtilizador.SUPERVISOR;
				}

				dr.Close();
			}
			catch(MySqlException ex)
			{
				//
				// tratar a excepção!!!!
				//
				System.Console.WriteLine("EXCEPÇÃO na validação do 'User': ");
				Console.WriteLine(ex);
				status = TiposUtilizador.NONE;
			}
			finally
			{
				//
				// fechar a conexão
				//
				DBHelper.closeConnection(conn);
			}
			
			return status;
		}
		
		public static string GetTipoCartao(string num_telefone) {
			string tipo = TipoCartao.PRE_PAGO_DBVALUE.ToString();
			MySqlConnection conn = DBHelper.getConnection();

			try
			{
				conn.Open();
				
				string sql = "select tipo_cartao from cartao where num_telemovel like ?num ;";
				MySqlCommand cmd = new MySqlCommand(sql,conn);
				cmd.Parameters.Add("?num",num_telefone);
				
				MySqlDataReader dr = cmd.ExecuteReader();
				
				if (dr.Read()) {
					tipo = dr.GetChar(0).ToString();
				}

				dr.Close();
			}
			catch(MySqlException ex)
			{
				//
				// tratar a excepção!!!!
				//
				System.Console.WriteLine("EXCEPÇÃO na obtençao do tipo de cartao: ");
				Console.WriteLine(ex);
			}
			finally
			{
				//
				// fechar a conexão
				//
				DBHelper.closeConnection(conn);
			}
			
			return tipo;
		}
		
		public static string GetEstadoCartao(string num_telefone) {
			string tipo = EstadoCartao.DESACTIVADO.ToString();
			MySqlConnection conn = DBHelper.getConnection();

			try
			{
				conn.Open();
				
				string sql = "select estado_cartao from cartao where num_telemovel like ?num ;";
				MySqlCommand cmd = new MySqlCommand(sql,conn);
				cmd.Parameters.Add("?num",num_telefone);
				
				MySqlDataReader dr = cmd.ExecuteReader();
				
				if (dr.Read()) {
					tipo = dr.GetChar(0).ToString();
				}

				dr.Close();
			}
			catch(MySqlException ex)
			{
				//
				// tratar a excepção!!!!
				//
				System.Console.WriteLine("EXCEPÇÃO na obtençao do tipo de cartao: ");
				Console.WriteLine(ex);
			}
			finally
			{
				//
				// fechar a conexão
				//
				DBHelper.closeConnection(conn);
			}
			
			return tipo;
		}
		
		public static int getIdForDayOfWeek(DayOfWeek dw) {
			int ret = 0;
			switch(dw) {
				case DayOfWeek.Sunday:
					ret = DiaSemana.Domingo;
					break;
				case DayOfWeek.Monday:
					ret = DiaSemana.Segunda;
					break;
				case DayOfWeek.Tuesday:
					ret = DiaSemana.Terca;
					break;
				case DayOfWeek.Wednesday:
					ret = DiaSemana.Quarta;
					break;
				case DayOfWeek.Thursday:
					ret = DiaSemana.Quinta;
					break;
				case DayOfWeek.Friday:
					ret = DiaSemana.Sexta;
					break;
				case DayOfWeek.Saturday:
					ret = DiaSemana.Sabado;
					break;
				default:
					ret = 0;
					break;
			}
			return ret;
		}
	}
}
