﻿/*
 * Created by SharpDevelop.
 * User: Marco José
 * Date: 08-01-2007
 * Time: 22:51
 */

using System;
using MySql.Data.MySqlClient;

namespace adav
{
	
	/// <summary>
	/// Description of Tarifario.
	/// </summary>
	public interface ITarifario
	{
		/// <summary>
		/// criacao de um tarifario (descricao)
		/// </summary>
		/// <param name="username"></param>
		/// <param name="password"></param>
		/// <param name="descricao"></param>
		/// <param name="comissao"></param>
		/// <returns></returns>
		OperationStatus CriarTarifario(string username, string password, string descricao, double comissao);
		
		/// <summary>
		/// criacao de um novo tarifario (detalhes do tarifario)
		/// </summary>
		/// <param name="rede_destino"></param>
		/// <param name="hora_inicio"></param>
		/// <param name="hora_fim"></param>
		/// <param name="custo_unidade"></param>
		/// <param name="unidades_minimas"></param>
		/// <returns> id tarifario </returns>
		OperationStatus CriarTarifarioDetalhe(string username, string password, int id_tarifario, int rede_destino, int dia_semana, int hora_inicio, int hora_fim, int unidade, double custo_unidade, int unidades_minimas);
		
		/// <summary>
		/// alteracao de um tarifario
		/// </summary>
		/// <param name="id_tarifario"></param>
		/// <param name="rede_destino"></param>
		/// <param name="hora_inicio"></param>
		/// <param name="hora_fim"></param>
		/// <param name="custo_unidade"></param>
		/// <param name="unidades_minimas"></param>
		/// <returns> id tarifario </returns>
		OperationStatus AlterarTarifarioDetalhe(string username, string password, int id_tarifario, int rede_destino, int dia_semana, int hora_inicio, int hora_fim, int unidade, double custo_unidade, int unidades_minimas);
	}
	
	
	/// <summary>
	/// Description of Tarifario.
	/// </summary>
	public class Tarifario : ITarifario 
	{
		public Tarifario()
		{
		}
		
		public OperationStatus CriarTarifario(
				string username, string password, 
				string descricao, double comissao)
		{
			OperationStatus status = DBHelper.ValidateUser(username,password);
			if(status != OperationStatus.OK)
				return status;
			
			//operadores nao podem criar tarifarios
			TiposUtilizador tipo = DBHelper.GetUserType(username,password);
			if(tipo == TiposUtilizador.OPERADOR)
				return OperationStatus.INVALID_OPERATION;

			MySqlConnection conn = DBHelper.getConnection();
			
			string dateString = DBHelper.getMySQLDateTime(DateTime.Now);
			string sql = "insert into tarifario(descricao,data_ultima_alteracao,comissao) " +
				"values (?dsc,?data,?comissao);";
			try {
				MySqlCommand cmd = new MySqlCommand(sql,conn);
//				cmd.CommandType = System.Data.CommandType.Text;
				cmd.Parameters.Add("?dsc",descricao);
				cmd.Parameters.Add("?data",dateString);
				cmd.Parameters.Add("?comissao",comissao);
				
				conn.Open();
				int exec = cmd.ExecuteNonQuery();
				status = OperationStatus.TARIFARIO_CRIADO;
			}
			catch(MySqlException ex) {
				System.Console.WriteLine("EXCEPÇÃO na inserção do 'Tarifário': ");
				Console.WriteLine(ex);
				status = OperationStatus.ERROR;
			}
			finally {
				DBHelper.closeConnection(conn);
			}
				
			return status;
		}

		public OperationStatus CriarTarifarioDetalhe(
				string username, string password, int id_tarifario, int rede_destino, 
				int dia_semana, int hora_inicio, int hora_fim, int unidade,
				double custo_unidade, int unidades_minimas)
		{
			OperationStatus status = DBHelper.ValidateUser(username,password);
			if(status != OperationStatus.OK)
				return status;
			
			// operadores nao podem alterar tarifarios
			TiposUtilizador tipo = DBHelper.GetUserType(username,password);
			if(tipo == TiposUtilizador.OPERADOR)
				return OperationStatus.INVALID_OPERATION;

			MySqlConnection conn = DBHelper.getConnection();
			
			string sql = "insert into tarifario_detalhe values (?id_tarif,?id_unidade,?rede,?id_periodo,?unidades_min,?custo_unidade);";
			try {
				MySqlCommand cmd = new MySqlCommand(sql,conn);
				cmd.Parameters.Add("?id_tarif",id_tarifario);
				cmd.Parameters.Add("?id_unidade",unidade);
				cmd.Parameters.Add("?rede",rede_destino);
				cmd.Parameters.Add("?id_periodo",DBHelper.getIDPeriodoByCiclo(conn,dia_semana,hora_inicio,hora_fim));
				cmd.Parameters.Add("?unidades_min",unidades_minimas);
				cmd.Parameters.Add("?custo_unidade",custo_unidade);
				
				conn.Open();
				int ret = cmd.ExecuteNonQuery();
				status = OperationStatus.TARIFARIO_CRIADO;
			}
			catch(MySqlException ex) {
				System.Console.WriteLine("EXCEPÇÃO na inserção do 'TarifárioDetalhe': ");
				Console.WriteLine(ex);
				status = OperationStatus.ERROR;
			}
			finally {
				DBHelper.closeConnection(conn);
			}
			
			return status;
		}
		
		public OperationStatus AlterarTarifarioDetalhe(
				string username, string password, int id_tarifario, int rede_destino, 
				int dia_semana, int hora_inicio, int hora_fim, int unidade, 
				double custo_unidade, int unidades_minimas)
		{
			OperationStatus status = DBHelper.ValidateUser(username,password);
			if(status != OperationStatus.OK)
				return status;
			
			// operadores nao podem alterar tarifarios
			TiposUtilizador tipo = DBHelper.GetUserType(username,password);
			if(tipo == TiposUtilizador.OPERADOR)
				return OperationStatus.INVALID_OPERATION;

			MySqlConnection conn = DBHelper.getConnection();
			
			string sql = "update tarifario_detalhe set custo_unidade = ?custo_unidade, unidades_minimas = ?unidades_min " +
				"where id_tarifario = ?id_tarif and id_unidade = ?id_unidade and id_rede = ?rede and id_periodo = ?id_periodo ;";
			try {
				MySqlCommand cmd = new MySqlCommand(sql,conn);
				cmd.Parameters.Add("?custo_unidade",custo_unidade);
				cmd.Parameters.Add("?unidades_min",unidades_minimas);
				
				cmd.Parameters.Add("?id_tarif",id_tarifario);
				cmd.Parameters.Add("?id_unidade",unidade);
				cmd.Parameters.Add("?rede",rede_destino);
				cmd.Parameters.Add("?id_periodo",DBHelper.getIDPeriodoByCiclo(conn,dia_semana,hora_inicio,hora_fim));
				
				conn.Open();
				int ret = cmd.ExecuteNonQuery();
				status = OperationStatus.TARIFARIO_ALTERADO;
			}
			catch(MySqlException ex) {
				System.Console.WriteLine("EXCEPÇÃO na alteração do 'TarifárioDetalhe': ");
				Console.WriteLine(ex);
				status = OperationStatus.ERROR;
			}
			finally {
				DBHelper.closeConnection(conn);
			}
			
			return status;
		}
		
	}
}
